## $Id: test_paypal.py 12060 2014-11-25 18:44:01Z uli $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
import shutil
import tempfile
import unittest
import paypalrestsdk
from zope.component import getGlobalSiteManager, queryUtility
from waeup.ikoba.interfaces import IPayPalConfig
from waeup.ikoba.payments.paypal import (
    get_paypal_config_file_path, parse_paypal_config, get_access_token,
    configure_sdk, get_payment,
    )
from waeup.ikoba.testing import (
    FunctionalLayer, FunctionalTestCase,
    )

#
# PayPal test config
#

EXTERNAL_PAYPAL_TESTS = False

#
# End of PayPal test config
#


def external_paypal_test(func):
    """A decorator that can block test functions.
    """
    if not EXTERNAL_PAYPAL_TESTS:
        myself = __file__
        if myself.endswith('.pyc'):
            myself = myself[:-2]
        print "WARNING: external paypal tests are skipped!"
        print "WARNING: edit %s to enable them." % myself
        return
    return func


class HelperTests(unittest.TestCase):

    def setUp(self):
        self.workdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.workdir)
        # unregister any remaining utils registered during tests
        util = queryUtility(IPayPalConfig)
        if util is not None:
            getGlobalSiteManager().unregisterUtility(util, IPayPalConfig)

    def test_get_paypal_config_file_path(self):
        # we can get a config file path (if registered)
        path = os.path.join(self.workdir, 'sample.cfg')
        getGlobalSiteManager().registerUtility(
            {'path': path, }, IPayPalConfig, '')
        result = get_paypal_config_file_path()
        assert result == path

    def test_get_paypal_config_file_path_no_file(self):
        # without IPayPalConfig util set, we get `None`
        result = get_paypal_config_file_path()
        assert result is None

    def test_parse_paypal_config(self):
        # we can parse paypal configs
        path = os.path.join(self.workdir, 'sample.cfg')
        open(path, 'w').write(
            "[rest-client]\n"
            "id = myid\n"
            "secret = mysecret\n"
            "mode = live\n")
        result = parse_paypal_config(path)
        assert result['client_id'] == u'myid'
        assert result['client_secret'] == u'mysecret'
        assert result['mode'] == 'live'

    def test_parse_paypal_config_defaults(self):
        # we have fallback values (defaults) in place
        path = os.path.join(self.workdir, 'sample.cfg')
        open(path, 'w').write(
            "[rest-client]\n"
            )
        result = parse_paypal_config(path)
        assert result['client_id'] is None
        assert result['client_secret'] is None
        assert result['mode'] == "sandbox"

    def test_configure_sdk(self):
        # we can configure the paypal sdk
        path = os.path.join(self.workdir, 'sample.cfg')
        open(path, 'w').write(
            "[rest-client]\n"
            "id = my-special-id\n"
            )
        result = configure_sdk(path)
        assert result['client_id'] == 'my-special-id'
        assert result['client_secret'] is None
        assert result['mode'] == "sandbox"
        assert paypalrestsdk.api.__api__.mode == 'sandbox'
        assert paypalrestsdk.api.__api__.client_id == 'my-special-id'


class FunctionalPaypalTests(FunctionalTestCase):

    layer = FunctionalLayer

    @external_paypal_test
    def test_get_access_token(self):
        # we can get an access token
        result = get_access_token()
        assert isinstance(result, unicode)

    @external_paypal_test
    def test_get_payment(self):
        # we can construct (paypal-)payment objects
        payment = get_payment()
        assert isinstance(payment, paypalrestsdk.Payment)
        result = payment.create()
        assert result is True
